@@ -3,8 +3,8 @@ package fileintegrity
 import (
 	"context"
 	"fmt"
+	"reflect"
 	"strconv"
-	"strings"
 
 	"github.com/go-logr/logr"
 
@@ -346,15 +346,13 @@ func (r *ReconcileFileIntegrity) Reconcile(request reconcile.Request) (reconcile
 			return reconcile.Result{}, common.IgnoreAlreadyExists(createErr)
 		}
 	} else {
-		// Handle an update of the daemon arguments when Debug and GracePeriod are changed. If they constitute a change
-		// then the daemonSet spec is updated and the pods are restarted.
-		updated, newArgs, err := updateDaemonSetPodArgs(daemonSet, instance)
-		if err != nil {
-			return reconcile.Result{}, nil
-		}
-		if updated {
-			dsCopy := daemonSet.DeepCopy()
-			dsCopy.Spec.Template.Spec.Containers[0].Args = newArgs
+		dsCopy := daemonSet.DeepCopy()
+		argsNeedUpdate := updateDSArgs(dsCopy, instance, reqLogger)
+		imgNeedsUpdate := updateDSImage(dsCopy, reqLogger)
+		nsNeedsUpdate := updateDSNodeSelector(dsCopy, instance, reqLogger)
+		tolsNeedsUpdate := updateDSTolerations(dsCopy, instance, reqLogger)
+
+		if argsNeedUpdate || imgNeedsUpdate || nsNeedsUpdate || tolsNeedsUpdate {
 			if err := r.client.Update(context.TODO(), dsCopy); err != nil {
 				return reconcile.Result{}, err
 			}
@@ -369,47 +367,54 @@ func (r *ReconcileFileIntegrity) Reconcile(request reconcile.Request) (reconcile
 	return reconcile.Result{}, nil
 }
 
-func intervalOpt(s string) bool {
-	return strings.HasPrefix(s, "--interval=")
+func updateDSNodeSelector(currentDS *appsv1.DaemonSet, fi *fileintegrityv1alpha1.FileIntegrity, logger logr.Logger) bool {
+	nsRef := &currentDS.Spec.Template.Spec.NodeSelector
+	expectedNS := fi.Spec.NodeSelector
+	needsUpdate := !reflect.DeepEqual(*nsRef, expectedNS)
+	if needsUpdate {
+		logger.Info("FileIntegrity needed nodeSelector update")
+		*nsRef = expectedNS
+	}
+	return needsUpdate
 }
 
-func debugOpt(s string) bool {
-	return strings.HasPrefix(s, "--debug=")
+func updateDSTolerations(currentDS *appsv1.DaemonSet, fi *fileintegrityv1alpha1.FileIntegrity, logger logr.Logger) bool {
+	tRef := &currentDS.Spec.Template.Spec.Tolerations
+	expectedTolerations := fi.Spec.Tolerations
+	needsUpdate := !reflect.DeepEqual(*tRef, expectedTolerations)
+	if needsUpdate {
+		logger.Info("FileIntegrity needed tolerations update")
+		*tRef = expectedTolerations
+	}
+	return needsUpdate
 }
 
-// Returns true with the daemon pod args from fi (for --interval and --debug options) if they differ from the current DS.
+// Returns true with the daemon pod args derived from the FileIntegrity object differ from the current DS.
 // Returns false if there was no difference.
-func updateDaemonSetPodArgs(currentDS *appsv1.DaemonSet, fi *fileintegrityv1alpha1.FileIntegrity) (bool, []string, error) {
-	var currentGracePeriod string
-	var currentDebug string
-
-	args := currentDS.Spec.Template.Spec.Containers[0].Args
-	newArgs := make([]string, 0)
-
-	for _, arg := range args {
-		if intervalOpt(arg) {
-			currentGracePeriod = arg[len("--interval="):]
-		} else if debugOpt(arg) {
-			currentDebug = arg[len("--debug="):]
-		}
-	}
-	if currentGracePeriod == "" || currentDebug == "" {
-		return false, newArgs, fmt.Errorf("bad daemon configuration")
+// If an update is needed, this will update the arguments from the given DaemonSet
+func updateDSArgs(currentDS *appsv1.DaemonSet, fi *fileintegrityv1alpha1.FileIntegrity, logger logr.Logger) bool {
+	argsRef := &currentDS.Spec.Template.Spec.Containers[0].Args
+	expectedArgs := daemonArgs(currentDS.Name, fi)
+	needsUpdate := !reflect.DeepEqual(*argsRef, expectedArgs)
+	if needsUpdate {
+		logger.Info("FileIntegrity needed DaemonSet command-line arguments update")
+		*argsRef = expectedArgs
 	}
+	return needsUpdate
+}
 
-	newGracePeriod := getGracePeriod(fi)
-	newDebug := getDebug(fi)
-	if currentGracePeriod != newGracePeriod || currentDebug != newDebug {
-		for _, arg := range args {
-			if !intervalOpt(arg) && !debugOpt(arg) {
-				newArgs = append(newArgs, arg)
-			}
-		}
-		newArgs = append(newArgs, fmt.Sprintf("--interval=%s", newGracePeriod))
-		newArgs = append(newArgs, fmt.Sprintf("--debug=%s", newDebug))
-		return true, newArgs, nil
+// Returns true with the daemon pod image differs from the current DS.
+// Returns false if there was no difference.
+// If an update is needed, this will update the image reference from the given DaemonSet
+func updateDSImage(currentDS *appsv1.DaemonSet, logger logr.Logger) bool {
+	currentImgRef := &currentDS.Spec.Template.Spec.Containers[0].Image
+	expectedImg := common.GetComponentImage(common.OPERATOR)
+	needsUpdate := *currentImgRef != expectedImg
+	if needsUpdate {
+		logger.Info("FileIntegrity needed image update", "Expected-Image", expectedImg, "Current-Image", currentImgRef)
+		*currentImgRef = expectedImg
 	}
-	return false, newArgs, nil
+	return needsUpdate
 }
 
 func defaultAIDEConfigMap(name string) *corev1.ConfigMap {
